@@ -17,18 +17,22 @@ class SitemapFacter(Facter):
     def get_fact_definitions(cls):
         return {
             'total.sitemap.indexes': {
-                'title': 'Total SiteMap indexes'
+                'title': 'Total SiteMap indexes',
+                'category': 'SEO',
             },
             'total.sitemap.urls': {
-                'title': 'Total SiteMap urls'
+                'title': 'Total SiteMap urls',
+                'category': 'SEO',
             },
             'total.size.sitemap': {
                 'title': 'Total SiteMap size',
-                'unit': 'kb'
+                'unit': 'kb',
+                'category': 'SEO',
             },
             'total.size.sitemap.gzipped': {
                 'title': 'Total SiteMap gzipped size',
-                'unit': 'kb'
+                'unit': 'kb',
+                'category': 'SEO',
             }
         }
 
@@ -67,12 +71,14 @@ class SitemapFacter(Facter):
         self.async_get(self.rebase('/robots.txt'), self.handle_robots_loaded)
 
     def get_sitemaps(self, response):
-        sitemaps = [self.rebase('/sitemap.xml')]
+        sitemaps = set([self.rebase('/sitemap.xml')])
 
         if response.status_code > 399:
             return sitemaps
 
-        return sitemaps + ROBOTS_SITEMAP.findall(response.text)
+        sitemaps.update(ROBOTS_SITEMAP.findall(response.text))
+
+        return sitemaps
 
     def handle_sitemap_loaded(self, url, response):
         logging.debug('Got sitemap %s with status %s' % (url, response.status_code))
