@@ -4,7 +4,6 @@
 import sys
 from uuid import uuid4
 from datetime import datetime, timedelta
-from random import choice
 import hashlib
 import logging
 
@@ -15,7 +14,7 @@ from ujson import dumps
 from tornado.concurrent import return_future
 
 from holmes.models import Base
-from holmes.utils import get_domain_from_url
+from holmes.utils import get_domain_from_url, url_ends_with_slash
 
 
 class Page(Base):
@@ -129,13 +128,10 @@ class Page(Base):
         lower_bound = (current_page - 1) * page_size
         upper_bound = lower_bound + page_size
 
-        expired_time = datetime.now() - timedelta(seconds=expiration)
+        expired_time = datetime.utcnow() - timedelta(seconds=expiration)
 
-        domains = db.query(Domain.id).filter(Domain.is_active).all()
-        active_domains = [item.id for item in domains]
-
-        if not active_domains:
-            return []
+        active_domains = Domain.get_active_domains(db)
+        active_domains_ids = [item.id for item in active_domains]
 
         pages_query = db \
             .query(
@@ -144,84 +140,103 @@ class Page(Base):
                 Page.score,
                 Page.last_review_date
             ) \
-            .filter(Page.last_review_date == None) \
-            .filter(Page.domain_id.in_(active_domains))
+            .filter(Page.domain_id.in_(active_domains_ids)) \
+            .filter(or_(
+                Page.last_review_date == None,
+                Page.last_review_date <= expired_time
+            )) \
+            .order_by(Page.score.desc())
 
-        pages_in_need_of_review = \
-            pages_query.order_by(Page.score.desc())[lower_bound:upper_bound]
-
-        if len(pages_in_need_of_review) == 0:
-            pages_query = db \
-                .query(
-                    Page.uuid,
-                    Page.url,
-                    Page.score,
-                    Page.last_review_date
-                ) \
-                .filter(Page.last_review_date <= expired_time) \
-                .filter(Page.domain_id.in_(active_domains))
-
-            pages_in_need_of_review = \
-                pages_query.order_by(Page.score.desc())[lower_bound:upper_bound]
-
-        return pages_in_need_of_review
+        return pages_query[lower_bound:upper_bound]
 
     @classmethod
     def get_next_jobs_count(cls, db, config):
         from holmes.models import Domain
 
         expiration = config.REVIEW_EXPIRATION_IN_SECONDS
-        expired_time = datetime.now() - timedelta(seconds=expiration)
+        expired_time = datetime.utcnow() - timedelta(seconds=expiration)
 
-        domains = db.query(Domain.id).filter(Domain.is_active).all()
-        active_domains = [item.id for item in domains]
+        active_domains = Domain.get_active_domains(db)
+        active_domains_ids = [item.id for item in active_domains]
 
-        without_review = db \
-            .query(sa.func.count(Page.id)) \
-            .filter(Page.last_review_date == None) \
-            .filter(Page.domain_id.in_(active_domains)) \
-            .scalar()
-
-        total = int(without_review)
-
-        if total == 0:
-            with_review = db \
-                .query(sa.func.count(Page.id)) \
-                .filter(Page.last_review_date <= expired_time) \
-                .filter(Page.domain_id.in_(active_domains)) \
+        return db \
+                .query(
+                    sa.func.count(Page.id)
+                ) \
+                .filter(Page.domain_id.in_(active_domains_ids)) \
+                .filter(or_(
+                    Page.last_review_date == None,
+                    Page.last_review_date <= expired_time
+                )) \
                 .scalar()
 
-            total = int(with_review)
-
-        return total
-
     @classmethod
-    def get_next_job(cls, db, expiration, cache, lock_expiration):
-        from holmes.models import Settings
+    def get_next_job(cls, db, expiration, cache, lock_expiration, avg_links_per_page=10):
+        from holmes.models import Worker, Domain, Limiter  # Avoid circular dependency
 
-        settings = Settings.instance(db)
+        page = None
+        lock = None
+        workers = db.query(Worker).all()
+        number_of_workers = len(workers)
 
-        pages_in_need_of_review = cls.get_next_job_list(db, expiration)
+        expired_time = datetime.utcnow() - timedelta(seconds=expiration)
 
-        if len(pages_in_need_of_review) == 0:
-            if settings.lambda_score > 0:
-                cls.update_pages_score_by(settings, settings.lambda_score, db)
+        active_domains = Domain.get_active_domains(db)
+        active_domains_ids = [item.id for item in active_domains]
+
+        all_domains_pages_in_need_of_review = {}
+
+        for domain_id in active_domains_ids:
+            pages = db \
+                .query(
+                    Page.uuid,
+                    Page.url,
+                    Page.score,
+                    Page.last_review_date
+                ) \
+                .filter(Page.domain_id == domain_id) \
+                .filter(or_(
+                    Page.last_review_date == None,
+                    Page.last_review_date <= expired_time
+                )) \
+                .order_by(Page.score.desc())[:number_of_workers]
+            if pages:
+                all_domains_pages_in_need_of_review[domain_id] = pages
+
+        pages_in_need_of_review = []
+        current_domain = 0
+        while all_domains_pages_in_need_of_review:
+            domains = all_domains_pages_in_need_of_review.keys()
+            if current_domain >= len(domains):
+                current_domain = 0
+
+            domain_id = domains[current_domain]
+
+            item = all_domains_pages_in_need_of_review[domain_id].pop(0)
+            pages_in_need_of_review.append(item)
+
+            if not all_domains_pages_in_need_of_review[domain_id]:
+                del all_domains_pages_in_need_of_review[domain_id]
+
+            current_domain += 1
+
+        if not pages_in_need_of_review:
             return None
 
-        if settings.lambda_score > pages_in_need_of_review[0].score:
-            cls.update_pages_score_by(settings, settings.lambda_score, db)
-            return None
+        for i in range(len(pages_in_need_of_review)):
+            if not Limiter.has_limit_to_work(db, active_domains, pages_in_need_of_review[i].url, avg_links_per_page):
+                continue
 
-        page = choice(pages_in_need_of_review)
+            lock = cache.has_next_job_lock(
+                pages_in_need_of_review[i].url,
+                lock_expiration
+            )
 
-        for i in range(10):
-            lock = cache.has_next_job_lock(page.url, lock_expiration)
-            if lock is None:
-                page = choice(pages_in_need_of_review)
-            else:
+            if lock is not None:
+                page = pages_in_need_of_review[i]
                 break
 
-        if lock is None:
+        if page is None:
             return None
 
         return {
@@ -231,13 +246,6 @@ class Page(Base):
             'lock': lock
         }
 
-    @classmethod
-    def update_pages_score_by(cls, settings, score, db):
-        settings.lambda_score = 0
-        page_count = cls.get_page_count(db)
-        individual_score = float(score) / float(page_count)
-        cls.update_scores(individual_score, db)
-
     @classmethod
     @return_future
     def add_page(cls, db, cache, url, score, fetch_method, publish_method, config, callback):
@@ -312,6 +320,7 @@ class Page(Base):
 
     @classmethod
     def insert_or_update_page(cls, url, score, domain, db, publish_method, cache):
+        url = url_ends_with_slash(url)
         url = url.encode('utf-8')
         url_hash = hashlib.sha512(url).hexdigest()
         page = Page.by_url_hash(url_hash, db)
